"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const CryptoJs = require("crypto-js");
const Https = require("https");
const SignalR = require("signalr-client");
const url_1 = require("url");
const Market_1 = require("./model/Market");
const Currency_1 = require("./model/Currency");
const Ticker_1 = require("./model/Ticker");
const MarketSummary_1 = require("./model/MarketSummary");
const Order_1 = require("./model/Order");
const OrderBookType_1 = require("./enum/OrderBookType");
const Trade_1 = require("./model/Trade");
const OrderType_1 = require("./enum/OrderType");
const OpenOrder_1 = require("./model/OpenOrder");
const Balance_1 = require("./model/Balance");
const ExchangeStateUpdate_1 = require("./model/ExchangeStateUpdate");
class BittrexApiClient {
    constructor(apiKey, apiSecret) {
        this.apiKey = apiKey;
        this.apiSecret = apiSecret;
    }
    getMarkets() {
        return this.makeRequest("public/getmarkets")
            .then((marketsJson) => {
            let markets = [];
            for (let marketJson of marketsJson) {
                markets.push(new Market_1.Market(marketJson));
            }
            return markets;
        })
            .catch((errorMessage) => {
            console.log("Error calling Bittrex API: " + errorMessage);
            return null;
        });
    }
    getCurrencies() {
        return this.makeRequest("public/getcurrencies")
            .then((currenciesJson) => {
            let currencies = [];
            for (let currencyJson of currenciesJson) {
                currencies.push(new Currency_1.Currency(currencyJson));
            }
            return currencies;
        })
            .catch((errorMessage) => {
            console.log("Error calling Bittrex API: " + errorMessage);
            return null;
        });
    }
    getTicker(market) {
        return this.makeRequest("public/getticker", ["market", market])
            .then((tickerJson) => {
            return new Ticker_1.Ticker(tickerJson);
        })
            .catch((errorMessage) => {
            console.log("Error calling Bittrex API: " + errorMessage);
            return null;
        });
    }
    getMarketSummaries() {
        return this.makeRequest("public/getmarketsummaries")
            .then((marketSummariesJson) => {
            let marketSummaries = [];
            for (let marketSummaryJson of marketSummariesJson) {
                marketSummaries.push(new MarketSummary_1.MarketSummary(marketSummaryJson));
            }
            return marketSummaries;
        })
            .catch((errorMessage) => {
            console.log("Error calling Bittrex API: " + errorMessage);
            return null;
        });
    }
    getMarketSummary(market) {
        return this.makeRequest("public/getmarketsummary", ["market", market])
            .then((marketSummaryJson) => {
            return new MarketSummary_1.MarketSummary(marketSummaryJson[0]);
        })
            .catch((errorMessage) => {
            console.log("Error calling Bittrex API: " + errorMessage);
            return null;
        });
    }
    getOrderBook(market, type) {
        return this.makeRequest("public/getorderbook", ["market", market], ["type", OrderBookType_1.OrderBookType[type]])
            .then((ordersJson) => {
            let orders = [];
            if (type === OrderBookType_1.OrderBookType.BOTH) {
                for (let buyOrderJson of ordersJson.buy) {
                    orders.push(new Order_1.Order(buyOrderJson, OrderType_1.OrderType.BUY));
                }
                for (let sellOrderJson of ordersJson.sell) {
                    orders.push(new Order_1.Order(sellOrderJson, OrderType_1.OrderType.SELL));
                }
                return orders;
            }
            for (let orderJson of ordersJson) {
                ordersJson.push(new Order_1.Order(orderJson, type === OrderBookType_1.OrderBookType.BUY ? OrderType_1.OrderType.BUY : OrderType_1.OrderType.SELL));
            }
            return orders;
        })
            .catch((errorMessage) => {
            console.log("Error calling Bittrex API: " + errorMessage);
            return null;
        });
    }
    getMarketHistory(market) {
        return this.makeRequest("/public/getmarkethistory", ["market", market])
            .then((tradesJson) => {
            let trades = [];
            for (let tradeJson of tradesJson) {
                trades.push(new Trade_1.Trade(tradeJson));
            }
            return trades;
        })
            .catch((errorMessage) => {
            console.log("Error calling Bittrex API: " + errorMessage);
            return null;
        });
    }
    buyLimit(market, quantity, rate) {
        return this.makeRequest("/market/buylimit", ["market", market], ["quantity", quantity.toString()], ["rate", rate.toString()])
            .then((orderIdJson) => {
            return orderIdJson.uuid;
        })
            .catch((errorMessage) => {
            console.log("Error calling Bittrex API: " + errorMessage);
            return null;
        });
    }
    cancel(orderId) {
        return this.makeRequest("/market/cancel", ["uuid", orderId])
            .then(() => {
            return true;
        })
            .catch((errorMessage) => {
            console.log("Error calling Bittrex API: " + errorMessage);
            return false;
        });
    }
    getOpenOrders(market) {
        return this.makeRequest("/market/getopenorders", ["market", market])
            .then((openOrdersJson) => {
            let openOrders = [];
            for (let openOrderJson of openOrdersJson) {
                openOrders.push(new OpenOrder_1.OpenOrder(openOrderJson));
            }
            return openOrders;
        })
            .catch((errorMessage) => {
            console.log("Error calling Bittrex API: " + errorMessage);
            return null;
        });
    }
    getBalances() {
        return this.makeRequest("/account/getbalances")
            .then((balancesJson) => {
            let balances = [];
            for (let balanceJson of balancesJson) {
                balances.push(new Balance_1.Balance(balanceJson));
            }
            return balances;
        })
            .catch((errorMessage) => {
            console.log("Error calling Bittrex API: " + errorMessage);
            return null;
        });
    }
    getBalance(currency) {
        return this.makeRequest("/account/getbalance", ["currency", currency])
            .then((balanceJson) => {
            return new Balance_1.Balance(balanceJson);
        })
            .catch((errorMessage) => {
            console.log("Error calling Bittrex API: " + errorMessage);
            return null;
        });
    }
    getDepositAddress(currency) {
        return this.makeRequest("/account/getdepositaddress", ["currency", currency])
            .then((depositAddressJson) => {
            return depositAddressJson.Address;
        })
            .catch((errorMessage) => {
            console.log("Error calling Bittrex API: " + errorMessage);
            return null;
        });
    }
    withdraw(currency, quantity, address, paymentId) {
        return this.makeRequest("/account/withdraw", ["currency", currency], ["quantity", quantity.toString()], ["address", address], ["paymentId", paymentId])
            .then((withdrawJson) => {
            return withdrawJson.uuid;
        })
            .catch((errorMessage) => {
            console.log("Error calling Bittrex API: " + errorMessage);
            return null;
        });
    }
    getExchangeStateUpdatesStream(watchableMarkets, callback) {
        let websocketClient = new SignalR.client(BittrexApiClient.WEB_SOCKET_HOST, [BittrexApiClient.WEB_SOCKET_HUB]);
        websocketClient.serviceHandlers.connected = () => {
            for (let watchableMarket of watchableMarkets) {
                websocketClient.call(BittrexApiClient.WEB_SOCKET_HUB, BittrexApiClient.WEB_SOCKET_HUB_SUBSCRIBE_DELTAS, watchableMarket);
            }
        };
        websocketClient.serviceHandlers.messageReceived = (messageJson) => {
            if (messageJson.type !== "utf8") {
                return;
            }
            messageJson = JSON.parse(messageJson.utf8Data);
            let updatesJson = messageJson.M;
            if (updatesJson === undefined || updatesJson.length === 0) {
                return;
            }
            let exchangeStateUpdates = [];
            for (let updateJson of updatesJson) {
                if (updateJson.M === "updateExchangeState") {
                    for (let exchangeStateUpdateJson of updateJson.A) {
                        exchangeStateUpdates.push(new ExchangeStateUpdate_1.ExchangeStateUpdate(exchangeStateUpdateJson));
                    }
                }
            }
            if (exchangeStateUpdates.length === 0) {
                return;
            }
            callback(exchangeStateUpdates);
        };
    }
    getAccounteUpdatesStream(callback) {
        let websocketClient = new SignalR.client(BittrexApiClient.WEB_SOCKET_HOST, [BittrexApiClient.WEB_SOCKET_HUB]);
        websocketClient.serviceHandlers.connected = () => {
            websocketClient.call(BittrexApiClient.WEB_SOCKET_HUB, "SubscribeToUserDeltas");
        };
        websocketClient.serviceHandlers.messageReceived = (messageJson) => {
            if (messageJson.type !== "utf8") {
                return;
            }
            messageJson = JSON.parse(messageJson.utf8Data);
            let updatesJson = messageJson.M;
            if (updatesJson === undefined || updatesJson.length === 0) {
                return;
            }
            let exchangeStateUpdates = [];
            for (let updateJson of updatesJson) {
                if (updateJson.M !== "updateExchangeState" && updateJson.M !== "updateSummaryState") {
                    return;
                }
                if (updateJson.M === "updateExchangeState") {
                    for (let exchangeStateUpdateJson of updateJson.A) {
                        exchangeStateUpdates.push(new ExchangeStateUpdate_1.ExchangeStateUpdate(exchangeStateUpdateJson));
                    }
                }
            }
            if (exchangeStateUpdates.length === 0) {
                return;
            }
            callback(exchangeStateUpdates);
        };
    }
    makeRequest(operation, ...parameters) {
        let apiEndpointUrl = new url_1.URL(BittrexApiClient.API_BASE_URL_PATH + operation, BittrexApiClient.API_BASE_URL_ROOT);
        apiEndpointUrl.searchParams.append(BittrexApiClient.API_API_KEY_PARAMETER, this.apiKey);
        apiEndpointUrl.searchParams.append(BittrexApiClient.API_TIMESTAMP_PARAMETER, new Date().getTime().toString());
        for (let parameter of parameters) {
            if (parameter === undefined || parameter === null) {
                continue;
            }
            apiEndpointUrl.searchParams.append(parameter[0], parameter[1]);
        }
        let apiSign = CryptoJs.HmacSHA512(apiEndpointUrl.toString(), this.apiSecret);
        return new Promise((fulfill, reject) => {
            let clientRequest = Https.request(apiEndpointUrl, (bittrexResponse) => {
                bittrexResponse.on("data", (bittrexData) => {
                    bittrexData = JSON.parse(bittrexData.toString());
                    if (bittrexData.success) {
                        fulfill(bittrexData.result);
                    }
                    else {
                        reject(bittrexData.message);
                    }
                });
            });
            clientRequest.setHeader("apisign", apiSign);
            clientRequest.end();
        });
    }
}
BittrexApiClient.WEB_SOCKET_HOST = "wss://socket.bittrex.com/signalr";
BittrexApiClient.WEB_SOCKET_HUB = "CoreHub";
BittrexApiClient.WEB_SOCKET_HUB_SUBSCRIBE_DELTAS = "SubscribeToExchangeDeltas";
BittrexApiClient.API_BASE_URL_PATH = "/api/v1.1/";
BittrexApiClient.API_BASE_URL_ROOT = "https://www.bittrex.com/";
BittrexApiClient.API_API_KEY_PARAMETER = "apikey";
BittrexApiClient.API_TIMESTAMP_PARAMETER = "nonce";
exports.BittrexApiClient = BittrexApiClient;
//# sourceMappingURL=BittrexApiClient.js.map